#! /usr/bin/env python
# -*- coding: utf8 -*-
#
# Markus Reinert, 06.10.12
# zeigt Untertitel zu einem Film in einem extra Fenster an
#
# Verwendung: "./Untertitel.py Film-Untertitel.srt <Zeit-Verschiebung in Sekunden>"
# SPACE für Pause
# ESC für Beenden

import pygame
import time
import sys

def timeFromString(string):
	t = string.split(":")
	sec = 0
	sec += int(t[0])*3600		# 1 Stunde = 3600 Sekunden
	sec += int(t[1])*60			# 1 Minute =   60 Sekunden
	sec += float(t[2].replace(",", "."))
	return sec

def stringFromTime(sec):
	sec = int(sec)
	t = ""
	h = str(sec/3600)
	sec = sec%3600
	if len(h) == 1:
		h = "0" + h
	t += h + ":"
	m = str(sec/60)
	sec = str(sec%60)
	if len(m) == 1:
		m = "0" + m
	if len(sec) == 1:
		sec = "0" + sec
	return t + m + ":" + sec

def pause():
	global offset
	start = time.time()
	text = pausefont.render("PAUSE", 1, (220, 0, 0))
	w, h = text.get_size()
	window.blit(text, (windowWidth/2.-w/2., windowHeight/2.-h/2.))
	pygame.display.flip()
	while True:
		for event in pygame.event.get():
			if event.type == pygame.QUIT:
				exit()
			if event.type == pygame.KEYDOWN:
				if event.key == pygame.K_ESCAPE:
					exit()
				if event.key == pygame.K_SPACE:
					end = time.time()
					offset -= (end-start)
					return

def draw(text, text2, now):
	window.fill((220, 220, 220))
	t = timefont.render(stringFromTime(now-STARTTIME+offset), 1, (50, 50, 170))
	window.blit(t, (10, 5))
	w = text.get_width()
	window.blit(text, (windowWidth/2.-w/2., 5))
	w2 = text2.get_width()
	window.blit(text2, (windowWidth/2.-w2/2., 47))
	pygame.display.flip()
	for event in pygame.event.get():
		if event.type == pygame.QUIT:
			exit()
		if event.type == pygame.KEYDOWN:
			if event.key == pygame.K_ESCAPE:
				exit()
			if event.key == pygame.K_SPACE:
				pause()

if len(sys.argv) > 1:
	path = sys.argv[1]
else:
	path = raw_input("Enter path to subtitle-file: ")

offset = 0
if len(sys.argv) > 2:
	offset = int(sys.argv[2])

# Untertitel einlesen
f = file(path, "r")
text = f.readlines()
f.close()

subtitle = []
for line in text:
	if line == "\r\n":
		i = text.index(line)
		subtitle.append(text[1:i])
		text = text[i+1:]

# initialisiere Pygame
pygame.init()

# Fenster
windowWidth = 1000
windowHeight = 92

window = pygame.display.set_mode((windowWidth, windowHeight))
pygame.display.set_caption("Untertitel - " + path)

# Schriftarten
try:	stfont = pygame.font.Font("/usr/share/fonts/truetype/ubuntu-font-family/Ubuntu-R.ttf", 35)			# Subtitle / Standard-Font
except IOError:	stfont = pygame.font.Font(None, 35)
try:	stIfont = pygame.font.Font("/usr/share/fonts/truetype/ubuntu-font-family/Ubuntu-RI.ttf", 35)		# Italic / Kursiv-Font
except IOError:	stIfont = pygame.font.Font(None, 35)
try:	timefont = pygame.font.Font("/usr/share/fonts/truetype/ubuntu-font-family/UbuntuMono-R.ttf", 14)	# Time / Zeit-Font
except IOError:	timefont = pygame.font.Font(None, 14)
try:	pausefont = pygame.font.Font("/usr/share/fonts/truetype/ubuntu-font-family/UbuntuMono-B.ttf", 40)	# Pause-Font
except IOError:	pausefont = pygame.font.Font(None, 40)
startfont = pygame.font.Font(None, 36)		# Start-Font

# Start-Bildschirm zeichnen
window.fill((220, 220, 220))
text = startfont.render("Press SPACE to start", 1, (50, 50, 50))
w, h = text.get_size()
window.blit(text, (windowWidth/2.-w/2., windowHeight/2.-h/2.))
pygame.display.flip()

# warten auf Leertaste
goon = False
while not goon:
	for event in pygame.event.get():
		if event.type == pygame.KEYDOWN:
			if event.key == pygame.K_SPACE:
				goon = True
				break

STARTTIME = time.time()

# Main-Loop: Untertitel anzeigen
for group in subtitle:
	text = stfont.render("", 1, (50, 50, 50))
	text2 = stfont.render("", 1, (50, 50, 50))
	t = group[0]
	start, end = t.split(" --> ")
	start = timeFromString(start)
	end = timeFromString(end)
	now = time.time()
	while not now-STARTTIME >= start-offset:
		now = time.time()
		draw(text, text2, now)
	c = 0		# Anzahl der Linien in einem Text; Zähler
	for line in group[1:]:
		if line.endswith("\n"):
			line = line[:-1]
		if line.endswith("\r"):
			line = line[:-1]
		if line.startswith("<i>"):
			line = line.replace("<i>", "").replace("</i>", "")
			if c == 0:
				text = stIfont.render(line, 1, (50, 50, 50))
			else:
				text2 = stIfont.render(line, 1, (50, 50, 50))
		else:
			if c == 0:
				text = stfont.render(line, 1, (50, 50, 50))
			else:
				text2 = stfont.render(line, 1, (50, 50, 50))
		c += 1
	while not now-STARTTIME >= end-offset:
		now = time.time()
		draw(text, text2, now)


